ColorConfiguratorListDialog = {
	CONTROLS = {
		"list",
		"textElement",
		"okButton",
		"noItemsText",
		"removeTextButton",
		"listSlider"
	},
	L10N_SYMBOL = {
		USER_COLORS = "colorConfigurator_ui_userColors",
		LAST_COLORS = "colorConfigurator_ui_lastColors",
		GAME_COLORS = "colorConfigurator_ui_gameColors",
		ADD_COLORS = "colorConfigurator_ui_addColors",
		INFO_REMOVE_COLOR = "colorConfigurator_ui_removeColor"
	},
	MAX_TEMP_COLORS = 30
}

local ColorConfiguratorListDialog_mt = Class(ColorConfiguratorListDialog, MessageDialog)

function ColorConfiguratorListDialog.new(target, custom_mt, colorConfigurator)
	local self = MessageDialog.new(target, custom_mt or ColorConfiguratorListDialog_mt)

	self:registerControls(ColorConfiguratorListDialog.CONTROLS)

	self.colorConfigurator = colorConfigurator

	self.disableOpenSound = true
	self.disableSearchOnClose = false
	self.textSample = nil

	self.listSections = {}

	self.tempColors = {}
	self.brandColors = {}
	self.addColors = {}

	self.wrapAround = false

	return self
end

function ColorConfiguratorListDialog:onMapLoad()
	self.brandColors = self:loadBrandColors()
	self.addColors = self:loadAdditionalColors(Utils.getFilename("colors.xml", self.colorConfigurator.baseDirectory), false)

	self:updateListData()
end

function ColorConfiguratorListDialog:onGuiSetupFinished()
	ColorConfiguratorListDialog:superClass().onGuiSetupFinished(self)

	self.textElement.customFocusSample = self.textSample
	ColorConfiguratorUtil.overwrittenFunction(self.list, "shouldFocusChange", self, "list_shouldFocusChange")
end

function ColorConfiguratorListDialog:onClose()
	if self.disableSearchOnClose then
		self.textElement:setText("")
		self:onTextChanged(self.textElement, "")
	end

	ColorConfiguratorListDialog:superClass().onClose(self)
end

function ColorConfiguratorListDialog:updateListData()
	self.listSections = {}

	local baseColors = self.colorConfigurator.baseConfigs
	local tempColors = self.tempColors
	local brandColors = self.brandColors
	local addColors = self.addColors

	local searchText = self.textElement.text

	if searchText ~= "" then
		searchText = utf8ToLower(searchText)

		local foundBaseColors = {}
		local foundTempColors = {}
		local foundBrandColors = {}
		local foundAddColors = {}

		for _, baseColor in pairs(baseColors) do
			if self:getCanAddItem(baseColor.name, searchText) then
				table.insert(foundBaseColors, baseColor)
			end
		end

		for _, tempColor in pairs(tempColors) do
			if self:getCanAddItem(tempColor.name, searchText) then
				table.insert(foundTempColors, tempColor)
			end
		end

		for _, brandColor in pairs(brandColors) do
			if self:getCanAddItem(brandColor.name, searchText) then
				table.insert(foundBrandColors, brandColor)
			end
		end

		for _, addColor in pairs(addColors) do
			if self:getCanAddItem(addColor.name, searchText) then
				table.insert(foundAddColors, addColor)
			end
		end

		baseColors = foundBaseColors
		tempColors = foundTempColors
		brandColors = foundBrandColors
		addColors = foundAddColors
	end

	if #baseColors > 0 then
		table.insert(self.listSections, {
			title = ColorConfiguratorListDialog.L10N_SYMBOL.USER_COLORS,
			colorData = baseColors,
			showButton = true
		})
	end

	if #tempColors > 0 then
		table.insert(self.listSections, {
			title = ColorConfiguratorListDialog.L10N_SYMBOL.LAST_COLORS,
			colorData = tempColors
		})
	end

	if #brandColors > 0 then
		table.insert(self.listSections, {
			title = ColorConfiguratorListDialog.L10N_SYMBOL.GAME_COLORS,
			colorData = brandColors
		})
	end

	if #addColors > 0 then
		table.insert(self.listSections, {
			title = ColorConfiguratorListDialog.L10N_SYMBOL.ADD_COLORS,
			colorData = addColors
		})
	end

	self.list:reloadData()
end

function ColorConfiguratorListDialog:getNumberOfSections()
	return #self.listSections
end

function ColorConfiguratorListDialog:getTitleForSectionHeader(list, sectionIndex)
	return string.format("%s (%s)", g_i18n:getText(self.listSections[sectionIndex].title), list.sections[sectionIndex].numItems)
end

function ColorConfiguratorListDialog:getNumberOfItemsInSection(list, sectionIndex)
	return #self.listSections[sectionIndex].colorData
end

function ColorConfiguratorListDialog:populateCellForItemInSection(list, sectionIndex, index, cell)
	local section = self.listSections[sectionIndex]
	local color = section.colorData[index]
	local name = color.name
	local r, g, b = unpack(color.color)
	local mat = Utils.getNoNil(color.material, 0)
	local isMetallic = ConfigurationUtil.isColorMetallic(mat)
	local isButtonVisible = section.showButton == true

	cell:getAttribute("title"):setText(name)
	cell:getAttribute("image"):setImageColor(nil, r, g, b, 1)
	cell:getAttribute("image"):setVisible(not isMetallic)
	cell:getAttribute("imageMetallic"):setImageColor(nil, r, g, b, 1)
	cell:getAttribute("imageMetallic"):setVisible(isMetallic)
	cell:getAttribute("button"):setVisible(isButtonVisible)
end

function ColorConfiguratorListDialog:onClickColor(list, sectionIndex, index)
	if self.list.totalItemCount > 0 then
		local sectionIndex = self.list.selectedSectionIndex
		local index = self.list.selectedIndex

		local section = self.listSections[sectionIndex]
		local color = section.colorData[index]
		local r, g, b = unpack(color.color)
		local mat = color.material

		self.colorConfigurator.dialog:setValues({r, g, b, Utils.getNoNil(mat, 0)}, mat ~= nil)
	end

	self:playSample(GuiSoundPlayer.SOUND_SAMPLES.CLICK)
	self:close()
end

function ColorConfiguratorListDialog:onClickRemoveColor(element)
	local listItem = element.parent.parent
	local sectionIndex = listItem.sectionIndex
	local index = listItem.indexInSection

	local section = self.listSections[sectionIndex]
	local color = section.colorData[index]

	g_gui:showYesNoDialog({
		text = string.format(g_i18n:getText(ColorConfiguratorListDialog.L10N_SYMBOL.INFO_REMOVE_COLOR), color.name),
		callback = function (yes)
			if yes then
				local success = self.colorConfigurator:removeBaseConfig(color)

				if success then
					self:updateListData()
				end
			end
		end
	})
end

function ColorConfiguratorListDialog:onClickRemoveText(element)
	self.textElement:setText("")
	self.textElement:setForcePressed(false)

	self:onTextChanged(self.textElement, "")
end

function ColorConfiguratorListDialog:addTempColor(colors, name)
	local color = {colors[1], colors[2], colors[3]}
	local material = colors[4]
	local tempColors = {}

	local i = 1

	for _, tempColor in pairs(self.tempColors) do
		if i < ColorConfiguratorListDialog.MAX_TEMP_COLORS then
			if tempColor.material ~= material or not ColorConfiguratorUtil.equals(tempColor.color, color) then
				table.insert(tempColors, tempColor)

				i = i + 1
			end
		else
			break
		end
	end

	table.insert(tempColors, {
		name = name,
		color = color,
		material = material
	})

	self.tempColors = tempColors

	self:updateListData()
end

function ColorConfiguratorListDialog:onTextChanged(element, text)
	self:updateListData()

	local totalItemCount = self.list.totalItemCount

	self.removeTextButton:setVisible(text ~= "")
	self.noItemsText:setVisible(totalItemCount == 0)
	self.okButton:setDisabled(totalItemCount == 0)

	if totalItemCount == 0 then
		self.listSlider:setVisible(false)
	end
end

function ColorConfiguratorListDialog:getCanAddItem(itemText, searchText)
	itemText = string.gsub(utf8ToLower(itemText), " ", "")
	searchText = string.gsub(searchText, " ", "")

	return string.find(itemText, searchText)
end

function ColorConfiguratorListDialog:loadBrandColors()
	local colorsList = {}

	for name, color in pairs(g_brandColorManager.brandColors) do
		local r, g, b, _ = unpack(color)

		table.insert(colorsList, {name = string.gsub(name, "_", " "), color = {r, g, b}})
	end

	table.sort(colorsList, function (a, b)
		return a.name < b.name
	end)

	return colorsList
end

function ColorConfiguratorListDialog:loadAdditionalColors(xmlFilename, printToLog)
	local colorsList = {}
	local xmlFile = XMLFile.load("colorsXML", xmlFilename)

	if xmlFile ~= nil then
		xmlFile:iterate("colors.color", function (_, key)
			local name = xmlFile:getString(key .. "#name")
			local color = xmlFile:getVector(key .. "#value")

			if name ~= nil and name:len() > 0 and color ~= nil then
				table.insert(colorsList, {name = name, color = color})
			end
		end)

		xmlFile:delete()

		ColorConfiguratorUtil.printOnLoad(printToLog, xmlFile)
	end

	table.sort(colorsList, function (a, b)
		return a.name < b.name
	end)

	return colorsList
end

function ColorConfiguratorListDialog:list_shouldFocusChange(list, superFunc, direction)
	local selectedIndex = list.selectedIndex
	local sectionIndex = list.selectedSectionIndex
	local retValue = superFunc(list, direction)

	if list.totalItemCount > 0 then
		local dir1 = FocusManager.TOP
		local dir2 = FocusManager.BOTTOM

		if direction == dir1 or direction == dir2 then
			local lastSectionIndex = #list.sections
			local lastIndex = list.sections[lastSectionIndex].numItems

			if direction == dir1 then
				if sectionIndex == 1 and selectedIndex == 1 then
					if self.wrapAround then
						list:setSelectedItem(lastSectionIndex, lastIndex, true, 0)
					end

					return false
				end
			else
				if sectionIndex == lastSectionIndex and selectedIndex == lastIndex then
					if self.wrapAround then
						list:setSelectedItem(1, 1, true, 0)
					end

					return false
				end
			end
		end
	end

	return retValue
end